% function [Status,InstrObj] = rs_ADS_connect(varargin) 
% *************************************************************************
% 
% Copyright: (c) 2005 Rohde & Schwarz GmbH & CO KG.    All rights reserved.
%                Muehldorfstr. 15
%                D-81671 Munich
%
% *************************************************************************
% This function returns an object for the remote instrument by means of the
% configuration information given by the user. Missing information is
% filled up with default values.
%%
% @param interface                        interface type (gpib, visa)
%   if interface is gpib:
%     @param ParamStruct.GPIB.vendor      vendor type (e.g. ni, agilent)
%     @param ParamStruct.GPIB.board       board number
%     @param ParamStruct.GPIB.primaddr    primary address
%     @param ParamStruct.GPIB.secaddr     secondary address (optional)
% if interface is visa:
%     @param ParamStruct.VISA.vendor      vendor type (e.g. ni, tek, agilent)
%     @param ParamStruct.VISA.rsc         VISA resource string
%%
% @return Status                      Status = 0, if object could not be created.
%                                     Status = 1, if object was created successfully.
% @return InstrObj                    The object which was created for the newly
%                                     connected remote instrument.        
%**************************************************************************

function [Status,InstrObj] = rs_ADS_connect (varargin)  

    global Status;
    Status   = 0;
    InstrObj = 0;

    % preset default values
    ParamStruct.interface     = 'visa';
    ParamStruct.GPIB.vendor   = 'ni';
    ParamStruct.GPIB.board    = 0;
    ParamStruct.GPIB.primaddr = 28;
    ParamStruct.GPIB.secaddr  = -1;
    ParamStruct.VISA.vendor   = 'ni';
    ParamStruct.VISA.rsc      = 'TCPIP::192.168.0.100::INSTR';

    % no arguments provided
    if (nargin<1)
        disp ('*** Not enough input prameters to rs_connect()');
        return;
    end

    % check if first argument is a string
    if (ischar(varargin{1}) ~=1)
        disp ('*** First parameter must be a string defining the interface type (''gpib'' or ''visa'').');
        return;
    end
    
    % get current info on supported instruments / interfaces
    HwInf = instrhwinfo();

    %
    % determine interface type
    %
    switch (upper(varargin{1}))
        case 'GPIB' 
            ParamStruct.interface = 'gpib';
        case 'VISA' 
            ParamStruct.interface = 'visa';
        otherwise
            disp ('*** Interface type must either be ''gpib'' or ''visa''.');
            return;
    end
    
    %
    % verify that the desired interface is supported
    %
    if (isempty(strmatch(ParamStruct.interface, HwInf.SupportedInterfaces, 'exact')))
        disp (['Supported interfaces are : ' HwInf.SupportedInterfaces]);
        disp (['Your selected interface  : ' ParamStruct.interface]);
        disp ( '*** Interface is not supported');
        return;
    end

    %
    % evaluate parameters of GPIB based interface
    %
    if (isequal(ParamStruct.interface,'gpib'))
        % vendor
        if (nargin > 1)
            switch(lower(varargin{2}))
                case 'advantech'
                    ParamStruct.GPIB.vendor = 'advantech';
                case 'agilent'
                    ParamStruct.GPIB.vendor = 'agilent';
                case 'cec'
                    ParamStruct.GPIB.vendor = 'cec';
                case 'contec'
                    ParamStruct.GPIB.vendor = 'contec';
                case 'ics'
                    ParamStruct.GPIB.vendor = 'ics';
                case 'iotech'
                    ParamStruct.GPIB.vendor = 'iotech';
                case 'keithley'
                    ParamStruct.GPIB.vendor = 'keithley';
                case 'mcc'
                    ParamStruct.GPIB.vendor = 'mcc';
                case 'ni'
                    ParamStruct.GPIB.vendor = 'ni';
                otherwise
                    disp ('*** GPIB interface type not supported');
                    return;
            end
        end
        % board number (range 0...7)
        if (nargin > 2)
            if (~isnumeric(varargin{3}))
                disp ('*** GPIB board number must be numeric.');
                return;
            end
            ParamStruct.GPIB.board = varargin{3};
            if (ParamStruct.GPIB.board < 0 || ParamStruct.GPIB.board > 7)
                disp ('*** GPIB board number is out of range. Use 0...7.');
                return;
            end
        end
        % set primary address of instrument (range 0...30)
        if (nargin > 3)
            if (~isnumeric(varargin{4}))
                error('*** GPIB primary address number must be numeric.');
            end
            ParamStruct.GPIB.primaddr = varargin{4};
            if (ParamStruct.GPIB.primaddr < 0 || ParamStruct.GPIB.primaddr > 30)
                disp ('*** GPIB primary address is out of range. Use 0...30.');
                return;
            end
        end
        % set secondary address of instrument (range 96...126)
        if (nargin > 4)
            if (~isnumeric(varargin{5}))
                disp ('*** GPIB secondary address number must be numeric.');
                return;
            end
            ParamStruct.GPIB.secaddr = varargin{5};
            if (ParamStruct.GPIB.secaddr < 96)
                disp ('*** GPIB secondary address is out of range. Minimum value is 96.');
                return;
            end
            if (ParamStruct.GPIB.secaddr > 126)
                disp ('*** GPIB secondary address is out of range. Maximum value is 126.');
                return;
            end
        end
        
        disp(['Interface      : ' ParamStruct.interface]);
        disp(['Vendor         : ' ParamStruct.GPIB.vendor]);
        disp(['Board No       : ' num2str(ParamStruct.GPIB.board)]);
        disp(['Primary addr   : ' num2str(ParamStruct.GPIB.primaddr)]);
        if (ParamStruct.GPIB.secaddr < 0)
            disp(['Secondary addr : not used']);
        else
            disp(['Secondary addr : ' num2str(ParamStruct.GPIB.secaddr)]);    
        end
        
        %
        % verify that the desired vendor is supported
        %
        HwInf = instrhwinfo(ParamStruct.interface);
        if (isempty(strmatch(ParamStruct.GPIB.vendor, HwInf.InstalledAdaptors, 'exact')))
            x = '';
            for N=1:length(HwInf.InstalledAdaptors)
                x = [HwInf.InstalledAdaptors{1,N} ' ' x];
            end
            disp(['Supported vendors are : ' x]);
            disp(['Your selected vendor  : ' ParamStruct.GPIB.vendor]);
            disp( '*** Vendor is not supported by your driver/Matlab installation.');
            return;
        end

       % build GPIB object and set Status variable
       [InstrObj, Status] = build_GPIB_object(ParamStruct);
    end
    
    % evaluate VISA resource string
    if (isequal(ParamStruct.interface,'visa'))
        % vendor
        if (nargin > 1)
            switch(lower(varargin{2}))
                case 'agilent'
                    ParamStruct.VISA.vendor = 'agilent';
                case 'tek'
                    ParamStruct.VISA.vendor = 'tek';
                case 'ni'
                    ParamStruct.VISA.vendor = 'ni';
                otherwise
                    disp ('*** VISA interface type not supported');
                    return;
            end
        end
        % resource string
        if (~ischar(varargin{3}))
            disp ('*** VISA resource string is not a string');
            return;
        end
        ParamStruct.VISA.rsc = varargin{3};
        
        %
        % verify that the desired vendor is supported
        %
        HwInf = instrhwinfo(ParamStruct.interface);
        if (isempty(strmatch(ParamStruct.VISA.vendor, HwInf.InstalledAdaptors, 'exact')))
           x = '';
            for N=1:length(HwInf.InstalledAdaptors)
                x = [HwInf.InstalledAdaptors{1,N} ' ' x];
            end
            disp (['Supported vendors are : ' x]);
            disp (['Your selected vendor  : ' ParamStruct.VISA.vendor]);
            disp ( '*** Vendor is not supported by your driver/Matlab installation.');
            return;
        end

        % build VISA object and set Status variable
        [InstrObj, Status] = build_VISA_object(ParamStruct);
    end
    
    if (Status < 1)
        disp ('*** VISA or GPIB object was not created.');
        return;
    end

    % set buffer sizes to 8 kBytes
    InstrObj.OutputBufferSize = 8*1024;
    InstrObj.InputBufferSize  = 8*1024;
    
    disp ('Device object created successfully.');
return;
    
% end of function rs_connect

%
% function - build a GPIB object
%
function [InstrObj, Status] = build_GPIB_object(ParamStruct)
    
    InstrObj = 0;
    Status = 0;
    
    % do we use a secondary address number ?
    if (ParamStruct.GPIB.secaddr >= 0)
        try
            % try to create GPIB object
            InstrObj = gpib (ParamStruct.GPIB.vendor, ParamStruct.GPIB.board, ParamStruct.GPIB.primaddr, 'SECOND', ParamStruct.GPIB.secaddr);
        catch
            disp ('*** Call to gpib() failed.');
            err = lasterror;
            disp (['*** Matlab error message : ' err.message]);
            return;
        end
        
        % try to open
        try
            fopen (InstrObj);
        catch
            disp (['*** Failed to open instrument connection. Return status is : ' InstrObj.Status]);
            return;
        end
        
        fclose(InstrObj);
        
    else

        try
            InstrObj = gpib(ParamStruct.GPIB.vendor, ParamStruct.GPIB.board, ParamStruct.GPIB.primaddr);
        catch
            disp ('*** Call to gpib() failed.');
            err = lasterror;
            disp (['*** Matlab error message : ' err.message]);
            return;
        end
        
        % try to open
        try
            fopen (InstrObj);
        catch
            disp (['*** Failed to open instrument connection. Return status is : ' InstrObj.Status]);
            return;
        end
        
        fclose(InstrObj);
    end
    Status = 1;
return;
    
%
% function - build a VISA object
%
function [InstrObj, Status] = build_VISA_object (ParamStruct)
    
    InstrObj = 0;
    Status = 0;
    
    try
        InstrObj = visa (ParamStruct.VISA.vendor, ParamStruct.VISA.rsc);             
    catch
        disp ('*** Call to visa() failed.');
        err = lasterror;
        disp (['*** Matlab error message : ' err.message]);
        return;
    end

    % try to open
    try
        fopen (InstrObj);
    catch
        disp (['*** Failed to open instrument connection. Status is ' InstrObj.Status]);
        return;
    end
        
    fclose(InstrObj);
    Status = 1;
return;
    
% end of file
