% *************************************************************************
% 
% Copyright: (c) 2011 Rohde & Schwarz GmbH & CO KG.    All rights reserved.
%                Muehldorfstr. 15
%                D-81671 Munich
%
%                8.3.2011       T. Roeder, Rohde&Schwarz
%
% This program demonstrates using the MATLAB toolkit for the
% purpose of generating and uploading an AWGN signal to the 
% vector signal generator.
%
% For instrument connection details please see Connect.m 
%
% *************************************************************************

% Use the VISA interface from National Instruments and
% connect via GPIB board number 0 to an instrument with address 28
% [status, InstrObject] = rs_connect( 'visa', 'ni', 'GPIB0::28::INSTR' );

% Use the VISA interface from National Instruments and
% connect via LAN (VXI-11)
  [status, InstrObject] = rs_connect( 'visa', 'ni', 'TCPIP::smbv100a255025::INSTR' );

% Use the raw TCP/IP socket connection on port 5025
% [status, InstrObject] = rs_connect( 'tcpip', 'smbv100a255025' );

if( ~status )
    clear;
    disp( 'rs_connect() failed.' );
    return;
end

% target instrument path for waveform file
%InstrTargetPath = 'D:\';                   % MS Windows based, e.g. SMU, SMJ, SMATE
%InstrTargetPath = '/var/smbv/';            % Linux based, flash memory, e.g. SMBV
 InstrTargetPath = '/hdd/';                 % Linux based, hard drive, e.g. SMBV 
%InstrTargetPath = '/var/user/';            % Linux based, SMW200A

StartARB         = 1;                       % start in path A
KeepLocalFile    = 0;                       % waveform only temporarily saved
LocalFileName    = 'awgn.wv';               % The local and remote file name

% *************************************************************************
% Create Signal
% *************************************************************************

% waveform parameters
Fsample              = 10e6;                % sample frequecy 10 MHz
Samples              = 1000000;             % 1M samples
DemonstratePureNoise = 1;                   % generate noise only - no signal

% set dependant variables
if DemonstratePureNoise==1
    AwgnAmplitude = 0.3;
    Offset        = 0.0;                    % no carrier
else
    AwgnAmplitude = 0.1;
    Offset        = 0.3;                    % carrier
end

% create numbers with gaussian distribution
Ran1 = rand( 1, Samples ) * 0.9998 + 0.0001;
Ran2 = rand( 1, Samples ) * 0.9998 + 0.0001;
Ran3 = rand( 1, Samples ) * 0.9998 + 0.0001;
Ran4 = rand( 1, Samples ) * 0.9998 + 0.0001;
am1  = AwgnAmplitude * sqrt( -2*log(Ran1) ) .* cos( 2*pi*Ran2 ) + Offset;
am2  = AwgnAmplitude * sqrt( -2*log(Ran3) ) .* cos( 2*pi*Ran4 ) + Offset;

% use digital filter from signal processing toolbox
h    = fdesign.lowpass( 'N,Fc', 100, 0.4 );
d    = design( h );                         % lowpass FIR filter
y1   = filter( d.Numerator, 1, am1 );       % conventional filtering
y2   = filter( d.Numerator, 1, am2 );       % conventional filtering

% *************************************************************************
% Setup Waveform Structure
% *************************************************************************

IQInfo.I_data         = y1;             % I signal
IQInfo.Q_data         = y2;             % Q signal
IQInfo.comment        = 'AWGN';         % optional comment
IQInfo.copyright      = 'Rohde&Schwarz';% optional copyright
IQInfo.clock          = Fsample;        % sample rate
IQInfo.no_scaling     = 0;              % scale automatically
IQInfo.path           = InstrTargetPath;% location on instrument     
IQInfo.filename       = LocalFileName;  % local and remore file name

% *************************************************************************
% Plot Data
% *************************************************************************

rs_visualize( Fsample, IQInfo.I_data, IQInfo.Q_data );

% *************************************************************************
% Instrument Setup
% *************************************************************************

% check for R&S device, we also need the *IDN? result later...
disp( 'Checking instrument...' );
[status, InstrIDN] = rs_send_query( InstrObject, '*IDN?' );
if( ~status ); clear;  return; end
if( isempty( strfind( InstrIDN, 'Rohde&Schwarz' ) ) )
    disp('This is not a Rohde&Schwarz device.');
    clear; return;
end

% reset the instrument
[status, OPCResponse] = rs_send_query( InstrObject, '*RST; *OPC?' );
if( ~status ); clear;  return; end
[status] = rs_send_command( InstrObject, '*CLS' );
if( ~status ); clear;  return; end

% generate and send waveform
[status] = rs_generate_wave( InstrObject, IQInfo, StartARB, KeepLocalFile );
if( ~status ); clear;  return; end

% Apply RF settings if this is not a pure baseband source
if( isempty( strfind( InstrIDN, 'AFQ' ) ) && isempty( strfind( InstrIDN, 'AMU' ) ) )

    disp('Setting up RF...');
        
    % apply some RF settings
    [status, Result] = rs_send_query( InstrObject, 'FREQ 1.0 GHz; *OPC?' );
    if( ~status ); clear;  return; end

    [status, Result] = rs_send_query( InstrObject, 'POW 0.0 dBm; *OPC?' );
    if( ~status ); clear;  return; end

    % switch output ON
    [status, Result] = rs_send_query( InstrObject, 'OUTP:STAT ON; *OPC?' );
    if( ~status ); clear;  return; end

end

% Read the instruments error queue
status = rs_check_instrument_errors( InstrObject );
if( ~status ); clear;  return; end

% delete instrument object
delete( InstrObject );

% clear variables
clear;

return;


