% *************************************************************************
% 
% Copyright: (c) 2011 Rohde & Schwarz GmbH & CO KG.    All rights reserved.
%                Muehldorfstr. 15
%                D-81671 Munich
%
%                8.3.2011       T. Roeder, Rohde&Schwarz
%
% This program demonstrates using the MATLAB toolkit for the
% purpose of generating and uploading an multi carrier signal to the 
% vector signal generator.
%
% For instrument connection details please see Connect.m 
%
% *************************************************************************

% Use the VISA interface from National Instruments and
% connect via GPIB board number 0 to an instrument with address 28
% [status, InstrObject] = rs_connect( 'visa', 'ni', 'GPIB0::28::INSTR' );

% Use the VISA interface from National Instruments and
% connect via LAN (VXI-11)
  [status, InstrObject] = rs_connect( 'visa', 'ni', 'TCPIP::smbv100a255025::INSTR' );

% Use the raw TCP/IP socket connection on port 5025
% [status, InstrObject] = rs_connect( 'tcpip', 'smbv100a255025' );

if( ~status )
    clear;
    disp( 'rs_connect() failed.' );
    return;
end

% target instrument path for waveform file
%InstrTargetPath = 'D:\';                   % MS Windows based, e.g. SMU, SMJ, SMATE
%InstrTargetPath = '/var/smbv/';            % Linux based, flash memory, e.g. SMBV
 InstrTargetPath = '/hdd/';                 % Linux based, hard drive, e.g. SMBV 
%InstrTargetPath = '/var/user/';            % Linux based, SMW200A

StartARB         = 1;                       % start in path A
KeepLocalFile    = 0;                       % waveform only temporarily saved
LocalFileName    = 'mc.wv';                 % The local and remote file name

% *************************************************************************
% Create Signal
% *************************************************************************

% waveform parameters
DFreq    = 10000;                           % 10 kHz spacing
Ncarrier = 1000;                            % 100 carriers
OS       = 10;                              % oversampling for max frequency

disp ('Settings:');
disp (['  Carrier Spacing : ', num2str(DFreq,5), ' Hz']);
disp (['  Oversampling    : ', num2str(OS,5)]);

% calculate highest frequency
Freq = Ncarrier * DFreq;
disp (['  Max. Frequency  : ', num2str(Freq,7), ' Hz']);

% calculate sample frequency
Fsample = Freq * OS;
Tsample = 1/Fsample;
disp (['  Sample Frequency: ', num2str(Fsample,8), ' Hz']);

% calculate number of samples
Nsamples = OS * Ncarrier;
disp (['  Samples         : ', num2str(Nsamples,5)]);
disp ( '  ' );

% Clock rate should not exceed 100 MHz
if( Fsample > 100e6 )
  error ('*** Error: Max. clock rate exceeded !');
  return;
end

% Samples count should be less than 16 Msamples
if( Nsamples > 16*1024*1024 )
  error ('*** Error: Max. sample count exceeded !');
  return;
end

% create array with frequencies
n = 1:1:Ncarrier;
FrequencyArray = 2.0*pi * DFreq * n;

% create a random start phase for each carrier
PhaseArray = rand( Ncarrier, 1 ) * 2 * pi;
% and build a matrix out of it
Phi = repmat( PhaseArray, 1, Nsamples );

% generate array of sample points for one period
k = 1:1:( OS*Ncarrier );
% and convert to time
Time = (k-1) * 1.0/Fsample;

% build matrix:  Ncarrier x OS*NCarrier
ResultMatrix = ( 1/Ncarrier ) * sin( FrequencyArray' * Time + Phi );

% sum up all carriers along the time dimension in our
% result matrix
I_data = sum( ResultMatrix, 1 );
Q_data = zeros( OS*Ncarrier, 1 )';

% scale to 1.0 for the max envelope 
MaxEnvelope = max( abs( complex( I_data, Q_data ) ) );
I_data = ( I_data / MaxEnvelope ) / sqrt(2);
Q_data = ( Q_data / MaxEnvelope ) / sqrt(2);

% *************************************************************************
% Setup Waveform Structure
% *************************************************************************

IQInfo.I_data         = I_data;         % I signal
IQInfo.Q_data         = Q_data;         % Q signal
IQInfo.comment        = 'Multi Carrier';% optional comment
IQInfo.copyright      = 'Rohde&Schwarz';% optional copyright
IQInfo.clock          = Fsample;        % sample rate
IQInfo.no_scaling     = 0;              % scale automatically
IQInfo.path           = InstrTargetPath;% location on instrument     
IQInfo.filename       = LocalFileName;  % local and remore file name

% *************************************************************************
% Plot Data
% *************************************************************************

rs_visualize( Fsample, IQInfo.I_data, IQInfo.Q_data );

% *************************************************************************
% Instrument Setup
% *************************************************************************

% check for R&S device, we also need the *IDN? result later...
disp( 'Checking instrument...' );
[status, InstrIDN] = rs_send_query( InstrObject, '*IDN?' );
if( ~status ); clear;  return; end
if( isempty( strfind( InstrIDN, 'Rohde&Schwarz' ) ) )
    disp('This is not a Rohde&Schwarz device.');
    clear; return;
end

% reset the instrument
[status, OPCResponse] = rs_send_query( InstrObject, '*RST; *OPC?' );
if( ~status ); clear;  return; end
[status] = rs_send_command( InstrObject, '*CLS' );
if( ~status ); clear;  return; end

% generate and send waveform
[status] = rs_generate_wave( InstrObject, IQInfo, StartARB, KeepLocalFile );
if( ~status ); clear;  return; end

% Apply RF settings if this is not a pure baseband source
if( isempty( strfind( InstrIDN, 'AFQ' ) ) && isempty( strfind( InstrIDN, 'AMU' ) ) )

    disp('Setting up RF...');
        
    % apply some RF settings
    [status, Result] = rs_send_query( InstrObject, 'FREQ 1.0 GHz; *OPC?' );
    if( ~status ); clear;  return; end

    [status, Result] = rs_send_query( InstrObject, 'POW 0.0 dBm; *OPC?' );
    if( ~status ); clear;  return; end

    % switch output ON
    [status, Result] = rs_send_query( InstrObject, 'OUTP:STAT ON; *OPC?' );
    if( ~status ); clear;  return; end

end

% Read the instruments error queue
status = rs_check_instrument_errors( InstrObject );
if( ~status ); clear;  return; end

% delete instrument object
delete( InstrObject );

% clear variables
clear;

return;
