% *************************************************************************
% 
% Copyright: (c) 2011 Rohde & Schwarz GmbH & CO KG.    All rights reserved.
%                Muehldorfstr. 15
%                D-81671 Munich
%
%                8.3.2011       T. Roeder, Rohde&Schwarz
%
% This program demonstrates using the MATLAB toolkit for the
% purpose of generating and uploading a pulse signal to the 
% vector signal generator.
%
% For instrument connection details please see Connect.m 
%
% *************************************************************************

% Use the VISA interface from National Instruments and
% connect via GPIB board number 0 to an instrument with address 28
% [status, InstrObject] = rs_connect( 'visa', 'ni', 'GPIB0::28::INSTR' );

% Use the VISA interface from National Instruments and
% connect via LAN (VXI-11)
  [status, InstrObject] = rs_connect( 'visa', 'ni', 'TCPIP::smbv100a255025::INSTR' );

% Use the raw TCP/IP socket connection on port 5025
% [status, InstrObject] = rs_connect( 'tcpip', 'smbv100a255025' );

if( ~status )
    clear;
    disp( 'rs_connect() failed.' );
    return;
end

% target instrument path for waveform file
%InstrTargetPath = 'D:\';                   % MS Windows based, e.g. SMU, SMJ, SMATE
%InstrTargetPath = '/var/smbv/';            % Linux based, flash memory, e.g. SMBV
 InstrTargetPath = '/hdd/';                 % Linux based, hard drive, e.g. SMBV 
%InstrTargetPath = '/var/user/';            % Linux based, SMW200A

StartARB         = 1;                       % start in path A
KeepLocalFile    = 0;                       % waveform only temporarily saved
LocalFileName    = 'pulse.wv';              % The local and remote file name

% *************************************************************************
% Create Signal
% *************************************************************************

% set the waveform parameters
Fsample       =  10e6;                % sample frequecy 10 MHz
Tdelay        =  10e-6;               % delay time before rising edge
Trise         =  10e-6;               % rise time
Ton           =  50e-6;               % on time
Tfall         =  10e-6;               % fall time
Toff          =  50e-6;               % off time after falling edge
phase         =    0.0;               % phase
level         =    1.0;               % signal level

% compute the time of one single sample
Tsample = 1/Fsample;

% compute required number of samples
Sdelay  = int32(Tdelay / Tsample);
Srise   = int32(Trise  / Tsample);
Son     = int32(Ton    / Tsample);
Sfall   = int32(Tfall  / Tsample);
Soff    = int32(Toff   / Tsample);

disp([ 'Samples for delay        : ' num2str(Sdelay)]);
disp([ 'Samples for rising edge  : ' num2str(Srise)]);
disp([ 'Samples for on period    : ' num2str(Son)]);
disp([ 'Samples for falling edge : ' num2str(Sfall)]);
disp([ 'Samples for off period   : ' num2str(Soff)]);

% build envelope for delay / on and off period
Edelay  = zeros( 1, Sdelay );
Eon     = ones( 1, Son );
Eoff    = zeros( 1, Soff );

% create points for rising and falling edge and also compute edge shape
Erise = [];
if( Srise>0 )
    Xrise = 0:1/double(Srise):1-1/double(Srise);
    Erise = 0.5 * (1.0 + sin(-pi/2+Xrise*pi));  
end

Efall = [];
if( Sfall>0 )
    Xfall = 0:1/double(Sfall):1-1/double(Sfall);
    Efall = 0.5 * (1.0 + cos(Xfall*pi));
end

% assemble the entire pulse waveform
PulseEnvelope = [Edelay Erise Eon Efall Eoff];

% create I and Q data
ArbI = level * sin( phase ) * PulseEnvelope;
ArbQ = level * cos( phase ) * PulseEnvelope;

% *************************************************************************
% Setup Waveform Structure
% *************************************************************************

IQInfo.I_data         = ArbI;
IQInfo.Q_data         = ArbQ;
IQInfo.comment        = 'Pulse';        % optional comment
IQInfo.copyright      = 'Rohde&Schwarz';% optional copyright
IQInfo.clock          = Fsample;        % sample rate
IQInfo.no_scaling     = 0;              % scale automatically
IQInfo.path           = InstrTargetPath;% location on instrument     
IQInfo.filename       = LocalFileName;  % local and remore file name
IQInfo.markerlist.one = [ [0 0]; ...
                          [Sdelay 1]; ...
                          [Sdelay+Srise+Son+Sfall 0] ];

% *************************************************************************
% Plot Data
% *************************************************************************

rs_visualize( Fsample, IQInfo.I_data, IQInfo.Q_data );

% *************************************************************************
% Instrument Setup
% *************************************************************************

% check for R&S device, we also need the *IDN? result later...
disp( 'Checking instrument...' );
[status, InstrIDN] = rs_send_query( InstrObject, '*IDN?' );
if( ~status ); clear;  return; end
if( isempty( strfind( InstrIDN, 'Rohde&Schwarz' ) ) )
    disp('This is not a Rohde&Schwarz device.');
    clear; return;
end

% reset the instrument
[status, OPCResponse] = rs_send_query( InstrObject, '*RST; *OPC?' );
if( ~status ); clear;  return; end
[status] = rs_send_command( InstrObject, '*CLS' );
if( ~status ); clear;  return; end

% generate and send waveform
[status] = rs_generate_wave( InstrObject, IQInfo, StartARB, KeepLocalFile );
if( ~status ); clear;  return; end

% Apply RF settings if this is not a pure baseband source
if( isempty( strfind( InstrIDN, 'AFQ' ) ) && isempty( strfind( InstrIDN, 'AMU' ) ) )

    disp('Setting up RF...');
        
    % apply some RF settings
    [status, Result] = rs_send_query( InstrObject, 'FREQ 1.0 GHz; *OPC?' );
    if( ~status ); clear;  return; end

    [status, Result] = rs_send_query( InstrObject, 'POW 0.0 dBm; *OPC?' );
    if( ~status ); clear;  return; end

    % switch output ON
    [status, Result] = rs_send_query( InstrObject, 'OUTP:STAT ON; *OPC?' );
    if( ~status ); clear;  return; end

end

% Read the instruments error queue
status = rs_check_instrument_errors( InstrObject );
if( ~status ); clear;  return; end

% delete instrument object
delete( InstrObject );

% clear variables
clear;

return;
