% *************************************************************************
% 
% Copyright: (c) 2011 Rohde & Schwarz GmbH & CO KG.    All rights reserved.
%                Muehldorfstr. 15
%                D-81671 Munich
%
%                8.3.2011       T. Roeder, Rohde&Schwarz
%
% This program demonstrates using the MATLAB toolkit for the
% purpose of generating and uploading an AWGN signal to the 
% vector signal generator.
%
% For instrument connection details please see Connect.m 
%
% *************************************************************************

% Use the VISA interface from National Instruments and
% connect via GPIB board number 0 to an instrument with address 28
% [status, InstrObject] = rs_connect( 'visa', 'ni', 'GPIB0::28::INSTR' );

% Use the VISA interface from National Instruments and
% connect via LAN (VXI-11)
  [status, InstrObject] = rs_connect( 'visa', 'ni', 'TCPIP::smbv100a255025::INSTR' );

% Use the raw TCP/IP socket connection on port 5025
% [status, InstrObject] = rs_connect( 'tcpip', 'smbv100a255025' );

if( ~status )
    clear;
    disp( 'rs_connect() failed.' );
    return;
end

% target instrument path for waveform file
%InstrTargetPath = 'D:\';                   % MS Windows based, e.g. SMU, SMJ, SMATE
%InstrTargetPath = '/var/smbv/';            % Linux based, flash memory, e.g. SMBV
 InstrTargetPath = '/hdd/';                 % Linux based, hard drive, e.g. SMBV 
%InstrTargetPath = '/var/user/';            % Linux based, SMW200A

StartARB         = 1;                       % start in path A
KeepLocalFile    = 0;                       % waveform only temporarily saved
LocalFileName    = 'awgn.wv';               % The local and remote file name

% *************************************************************************
% Create Signal
% *************************************************************************

% waveform parameters
DFreq1   = 2e6;                             % 2 MHz
DFreq2   = 5e6;                             % 5 MHz 
OS       = 12;                              % oversampling for max frequency

disp ('Settings:');
disp (['  F1              : ', num2str(DFreq1,5), ' Hz']);
disp (['  F2              : ', num2str(DFreq2,5), ' Hz']);
disp (['  Oversampling    : ', num2str(OS,5)]);
disp ( '  ' );

% get the highest signal frequency
FMax = DFreq1;
if( DFreq2>FMax )
    FMax = DFreq2;
end

% calculate clock rate for ARB
Fsample = FMax * OS;
disp( ['  Clock Rate      : ', num2str(Fsample/1e6,5), ' MHz'] );
if( Fsample > 300e6)
  error ('*** Error: Max. clock rate of 300 MHz exceeded !');
  return;
end

%ggT of the two frequencies
a = int32(DFreq1);
b = int32(DFreq2);
if( a>0 && b>0 )
    while( a ~= b )
        if( a>b )
            a = a-b;
        else
            b = b-a;
        end
    end
end
iggt = a;

disp( ['  ggT             : ', num2str(iggt), ' Hz' ] );

% useful period width
Tggt = 1.0 / double(iggt);
disp( ['  Signal Duration : ', num2str(Tggt,5), ' s'] );

% calculate number of samples
Nsamples = Tggt * Fsample;
disp( ['  Samples         : ', num2str(Nsamples,5)] );
% Make sure to use less than 16 Msamples
if( Nsamples > 16*1024*1024 )
  error ('*** Error: Maximum number of samples exceeded !');
  return;
end

disp (['  ']);

% generate array of sample points
k = 1:1:Nsamples;
% and convert to time scale
Time = (k-1) * 1.0/Fsample;

% signal 1 and 2
Sig1 = cos( 2.0 * pi * DFreq1 * Time );
Sig2 = cos( 2.0 * pi * DFreq2 * Time );

Sig3 = sin( 2.0 * pi * DFreq1 * Time );
Sig4 = sin( 2.0 * pi * DFreq2 * Time );

% sum up all carriers along the time dimension in our
% result matrix
I_data = (Sig1 + Sig2);
Q_data = (Sig3 + Sig4);

% scale to 1.0 for the max envelope 
MaxEnvelope = max( abs(complex(I_data,Q_data)) );
I_data = (I_data / MaxEnvelope) / sqrt(2);
Q_data = (Q_data / MaxEnvelope) / sqrt(2);

% *************************************************************************
% Setup Waveform Structure
% *************************************************************************

IQInfo.I_data         = I_data;         % I signal
IQInfo.Q_data         = Q_data;         % Q signal
IQInfo.comment        = 'Two Tone';     % optional comment
IQInfo.copyright      = 'Rohde&Schwarz';% optional copyright
IQInfo.clock          = Fsample;        % sample rate
IQInfo.no_scaling     = 0;              % scale automatically
IQInfo.path           = InstrTargetPath;% location on instrument     
IQInfo.filename       = LocalFileName;  % local and remore file name

% *************************************************************************
% Plot Data
% *************************************************************************

rs_visualize( Fsample, IQInfo.I_data, IQInfo.Q_data );

% *************************************************************************
% Instrument Setup
% *************************************************************************

% check for R&S device, we also need the *IDN? result later...
disp( 'Checking instrument...' );
[status, InstrIDN] = rs_send_query( InstrObject, '*IDN?' );
if( ~status ); clear;  return; end
if( isempty( strfind( InstrIDN, 'Rohde&Schwarz' ) ) )
    disp('This is not a Rohde&Schwarz device.');
    clear; return;
end

% reset the instrument
[status, OPCResponse] = rs_send_query( InstrObject, '*RST; *OPC?' );
if( ~status ); clear;  return; end
[status] = rs_send_command( InstrObject, '*CLS' );
if( ~status ); clear;  return; end

% generate and send waveform
[status] = rs_generate_wave( InstrObject, IQInfo, StartARB, KeepLocalFile );
if( ~status ); clear;  return; end

% Apply RF settings if this is not a pure baseband source
if( isempty( strfind( InstrIDN, 'AFQ' ) ) && isempty( strfind( InstrIDN, 'AMU' ) ) )

    disp( 'Setting up RF...' );
        
    % apply some RF settings
    [status, Result] = rs_send_query( InstrObject, 'FREQ 1.0 GHz; *OPC?' );
    if( ~status ); clear;  return; end

    [status, Result] = rs_send_query( InstrObject, 'POW 0.0 dBm; *OPC?' );
    if( ~status ); clear;  return; end

    % switch output ON
    [status, Result] = rs_send_query( InstrObject, 'OUTP:STAT ON; *OPC?' );
    if( ~status ); clear;  return; end

end

% Read the instruments error queue
status = rs_check_instrument_errors( InstrObject );
if( ~status ); clear;  return; end

% delete instrument object
delete( InstrObject );

% clear variables
clear;

return;







